# coding: utf-8

# flake8: noqa
"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from cdp.openapi_client.models.create_evm_account_request import CreateEvmAccountRequest
from cdp.openapi_client.models.create_evm_smart_account_request import CreateEvmSmartAccountRequest
from cdp.openapi_client.models.create_policy_request import CreatePolicyRequest
from cdp.openapi_client.models.create_solana_account_request import CreateSolanaAccountRequest
from cdp.openapi_client.models.eip712_domain import EIP712Domain
from cdp.openapi_client.models.eip712_message import EIP712Message
from cdp.openapi_client.models.error import Error
from cdp.openapi_client.models.error_type import ErrorType
from cdp.openapi_client.models.eth_value_criterion import EthValueCriterion
from cdp.openapi_client.models.evm_account import EvmAccount
from cdp.openapi_client.models.evm_address_criterion import EvmAddressCriterion
from cdp.openapi_client.models.evm_call import EvmCall
from cdp.openapi_client.models.evm_network_criterion import EvmNetworkCriterion
from cdp.openapi_client.models.evm_smart_account import EvmSmartAccount
from cdp.openapi_client.models.evm_user_operation import EvmUserOperation
from cdp.openapi_client.models.import_evm_account_request import ImportEvmAccountRequest
from cdp.openapi_client.models.list_evm_accounts200_response import ListEvmAccounts200Response
from cdp.openapi_client.models.list_evm_smart_accounts200_response import ListEvmSmartAccounts200Response
from cdp.openapi_client.models.list_evm_token_balances200_response import ListEvmTokenBalances200Response
from cdp.openapi_client.models.list_evm_token_balances_network import ListEvmTokenBalancesNetwork
from cdp.openapi_client.models.list_policies200_response import ListPolicies200Response
from cdp.openapi_client.models.list_response import ListResponse
from cdp.openapi_client.models.list_solana_accounts200_response import ListSolanaAccounts200Response
from cdp.openapi_client.models.policy import Policy
from cdp.openapi_client.models.prepare_user_operation_request import PrepareUserOperationRequest
from cdp.openapi_client.models.request_evm_faucet200_response import RequestEvmFaucet200Response
from cdp.openapi_client.models.request_evm_faucet_request import RequestEvmFaucetRequest
from cdp.openapi_client.models.request_solana_faucet200_response import RequestSolanaFaucet200Response
from cdp.openapi_client.models.request_solana_faucet_request import RequestSolanaFaucetRequest
from cdp.openapi_client.models.rule import Rule
from cdp.openapi_client.models.send_evm_transaction200_response import SendEvmTransaction200Response
from cdp.openapi_client.models.send_evm_transaction_criteria_inner import SendEvmTransactionCriteriaInner
from cdp.openapi_client.models.send_evm_transaction_request import SendEvmTransactionRequest
from cdp.openapi_client.models.send_evm_transaction_rule import SendEvmTransactionRule
from cdp.openapi_client.models.send_user_operation_request import SendUserOperationRequest
from cdp.openapi_client.models.sign_evm_hash200_response import SignEvmHash200Response
from cdp.openapi_client.models.sign_evm_hash_request import SignEvmHashRequest
from cdp.openapi_client.models.sign_evm_message200_response import SignEvmMessage200Response
from cdp.openapi_client.models.sign_evm_message_request import SignEvmMessageRequest
from cdp.openapi_client.models.sign_evm_transaction200_response import SignEvmTransaction200Response
from cdp.openapi_client.models.sign_evm_transaction_criteria_inner import SignEvmTransactionCriteriaInner
from cdp.openapi_client.models.sign_evm_transaction_request import SignEvmTransactionRequest
from cdp.openapi_client.models.sign_evm_transaction_rule import SignEvmTransactionRule
from cdp.openapi_client.models.sign_evm_typed_data200_response import SignEvmTypedData200Response
from cdp.openapi_client.models.sign_sol_transaction_criteria_inner import SignSolTransactionCriteriaInner
from cdp.openapi_client.models.sign_sol_transaction_rule import SignSolTransactionRule
from cdp.openapi_client.models.sign_solana_message200_response import SignSolanaMessage200Response
from cdp.openapi_client.models.sign_solana_message_request import SignSolanaMessageRequest
from cdp.openapi_client.models.sign_solana_transaction200_response import SignSolanaTransaction200Response
from cdp.openapi_client.models.sign_solana_transaction_request import SignSolanaTransactionRequest
from cdp.openapi_client.models.sol_address_criterion import SolAddressCriterion
from cdp.openapi_client.models.solana_account import SolanaAccount
from cdp.openapi_client.models.token import Token
from cdp.openapi_client.models.token_amount import TokenAmount
from cdp.openapi_client.models.token_balance import TokenBalance
from cdp.openapi_client.models.update_evm_account_request import UpdateEvmAccountRequest
from cdp.openapi_client.models.update_policy_request import UpdatePolicyRequest
from cdp.openapi_client.models.update_solana_account_request import UpdateSolanaAccountRequest



import contextlib
import functools
import hashlib
import inspect
import json
import os
import time
import traceback

import requests
from pydantic import BaseModel

from cdp.__version__ import __version__
from cdp.openapi_client.errors import ApiError

# This is a public client id for the analytics service
public_client_id = "54f2ee2fb3d2b901a829940d70fbfc13"


class ErrorEventData(BaseModel):
    """The data in an error event."""

    method: str  # The API method where the error occurred, e.g. createAccount, getAccount
    message: str  # The error message
    name: str  # The name of the event. This should match the name in AEC
    stack: str | None = None  # The error stack trace


EventData = ErrorEventData

Analytics = {
    "identifier": "",  # set in cdp_client.py
}


async def send_event(event: EventData) -> None:
    """Send an analytics event to the default endpoint.

    Args:
        event: The event data containing event-specific fields

    Returns:
        None - resolves when the event is sent

    """
    if os.getenv("DISABLE_CDP_ERROR_REPORTING") == "true":
        return

    timestamp = int(time.time() * 1000)

    enhanced_event = {
        "user_id": Analytics["identifier"],
        "event_type": event.name,
        "platform": "server",
        "timestamp": timestamp,
        "event_properties": {
            "project_name": "cdp-sdk",
            "cdp_sdk_language": "python",
            "version": __version__,
            **event.model_dump(),
        },
    }

    events = [enhanced_event]
    stringified_event_data = json.dumps(events)
    upload_time = str(timestamp)

    checksum = hashlib.md5((stringified_event_data + upload_time).encode("utf-8")).hexdigest()

    analytics_service_data = {
        "client": public_client_id,
        "e": stringified_event_data,
        "checksum": checksum,
    }

    api_endpoint = "https://cca-lite.coinbase.com"
    event_path = "/amp"
    event_endpoint = f"{api_endpoint}{event_path}"

    requests.post(
        event_endpoint,
        headers={"Content-Type": "application/json"},
        json=analytics_service_data,
    )


def wrap_with_error_tracking(func):
    """Wrap a method with error tracking.

    Args:
        func: The function to wrap.

    Returns:
        The wrapped function.

    """

    @functools.wraps(func)
    async def wrapper(*args, **kwargs):
        try:
            return await func(*args, **kwargs)
        except Exception as error:
            if not should_track_error(error):
                raise error

            event_data = ErrorEventData(
                method=func.__name__,
                message=str(error),
                stack=traceback.format_exc(),
                name="error",
            )

            with contextlib.suppress(Exception):
                await send_event(event_data)

            raise error

    return wrapper


def wrap_class_with_error_tracking(cls):
    """Wrap all methods of a class with error tracking.

    Args:
        cls: The class to wrap.

    Returns:
        The class with wrapped methods.

    """
    if os.getenv("DISABLE_CDP_ERROR_REPORTING") == "true":
        return cls

    for name, method in inspect.getmembers(cls, inspect.isfunction):
        if not name.startswith("__"):
            setattr(cls, name, wrap_with_error_tracking(method))
    return cls


def should_track_error(error: Exception) -> bool:
    """Determine if an error should be tracked.

    Args:
        error: The error to check.

    Returns:
        True if the error should be tracked, False otherwise.

    """
    return isinstance(error, Exception) and not isinstance(error, ApiError)



# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from cdp.openapi_client.models.evm_call import EvmCall
from typing import Optional, Set
from typing_extensions import Self

class EvmUserOperation(BaseModel):
    """
    EvmUserOperation
    """ # noqa: E501
    network: StrictStr = Field(description="The network the user operation is for.")
    user_op_hash: Annotated[str, Field(strict=True)] = Field(description="The hash of the user operation. This is not the transaction hash, as a transaction consists of multiple user operations. The user operation hash is the hash of this particular user operation which gets signed by the owner of the Smart Account.", alias="userOpHash")
    calls: List[EvmCall] = Field(description="The list of calls in the user operation.")
    status: StrictStr = Field(description="The status of the user operation.")
    transaction_hash: Optional[Annotated[str, Field(strict=True)]] = Field(default=None, description="The hash of the transaction that included this particular user operation. This gets set after the user operation is broadcasted and the transaction is included in a block.", alias="transactionHash")
    __properties: ClassVar[List[str]] = ["network", "userOpHash", "calls", "status", "transactionHash"]

    @field_validator('network')
    def network_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['base-sepolia', 'base']):
            raise ValueError("must be one of enum values ('base-sepolia', 'base')")
        return value

    @field_validator('user_op_hash')
    def user_op_hash_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^0x[0-9a-fA-F]{64}$", value):
            raise ValueError(r"must validate the regular expression /^0x[0-9a-fA-F]{64}$/")
        return value

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['pending', 'signed', 'broadcast', 'complete', 'failed']):
            raise ValueError("must be one of enum values ('pending', 'signed', 'broadcast', 'complete', 'failed')")
        return value

    @field_validator('transaction_hash')
    def transaction_hash_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^0x[0-9a-fA-F]{64}$|^$", value):
            raise ValueError(r"must validate the regular expression /^0x[0-9a-fA-F]{64}$|^$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of EvmUserOperation from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in calls (list)
        _items = []
        if self.calls:
            for _item_calls in self.calls:
                if _item_calls:
                    _items.append(_item_calls.to_dict())
            _dict['calls'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of EvmUserOperation from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "network": obj.get("network"),
            "userOpHash": obj.get("userOpHash"),
            "calls": [EvmCall.from_dict(_item) for _item in obj["calls"]] if obj.get("calls") is not None else None,
            "status": obj.get("status"),
            "transactionHash": obj.get("transactionHash")
        })
        return _obj





from cdp.openapi_client.models.eth_value_criterion import EthValueCriterion
from cdp.openapi_client.models.evm_address_criterion import EvmAddressCriterion
from cdp.openapi_client.models.evm_network_criterion import EvmNetworkCriterion
from cdp.openapi_client.models.rule import Rule
from cdp.openapi_client.models.send_evm_transaction_criteria_inner import (
    SendEvmTransactionCriteriaInner,
)
from cdp.openapi_client.models.send_evm_transaction_rule import SendEvmTransactionRule
from cdp.openapi_client.models.sign_evm_transaction_criteria_inner import (
    SignEvmTransactionCriteriaInner,
)
from cdp.openapi_client.models.sign_evm_transaction_rule import SignEvmTransactionRule
from cdp.openapi_client.models.sign_sol_transaction_criteria_inner import (
    SignSolTransactionCriteriaInner,
)
from cdp.openapi_client.models.sign_sol_transaction_rule import SignSolTransactionRule
from cdp.openapi_client.models.sol_address_criterion import SolAddressCriterion
from cdp.policies.types import (
    EthValueCriterion as EthValueCriterionModel,
    EvmAddressCriterion as EvmAddressCriterionModel,
    EvmNetworkCriterion as EvmNetworkCriterionModel,
    Rule as RuleType,
    SendEvmTransactionRule as SendEvmTransactionRuleModel,
    SignEvmTransactionRule as SignEvmTransactionRuleModel,
    SignSolanaTransactionRule as SignSolanaTransactionRuleModel,
    SolanaAddressCriterion as SolanaAddressCriterionModel,
)


def map_request_rules_to_openapi_format(request_rules: list[RuleType]) -> list[Rule]:
    """Build a properly formatted list of OpenAPI policy rules from a list of request rules.

    Args:
        request_rules (List[RuleType]): The request rules to build from.

    Returns:
        List[Rule]: A list of rules formatted for the OpenAPI policy.

    """
    rules = []
    for rule in request_rules:
        if rule.operation == "sendEvmTransaction":
            criteria = []
            for criterion in rule.criteria:
                if criterion.type == "ethValue":
                    criteria.append(
                        SendEvmTransactionCriteriaInner(
                            actual_instance=EthValueCriterion(
                                eth_value=criterion.ethValue,
                                operator=criterion.operator,
                                type="ethValue",
                            )
                        )
                    )
                elif criterion.type == "evmAddress":
                    criteria.append(
                        SendEvmTransactionCriteriaInner(
                            actual_instance=EvmAddressCriterion(
                                addresses=criterion.addresses,
                                operator=criterion.operator,
                                type="evmAddress",
                            )
                        )
                    )
                elif criterion.type == "evmNetwork":
                    criteria.append(
                        SendEvmTransactionCriteriaInner(
                            actual_instance=EvmNetworkCriterion(
                                networks=criterion.networks,
                                operator=criterion.operator,
                                type="evmNetwork",
                            )
                        )
                    )
                else:
                    raise ValueError(
                        f"Unknown criterion type {criterion.type} for operation {rule.operation}"
                    )

            rules.append(
                Rule(
                    actual_instance=SendEvmTransactionRule(
                        action=rule.action,
                        operation="sendEvmTransaction",
                        criteria=criteria,
                    )
                )
            )
        elif rule.operation == "signEvmTransaction":
            criteria = []
            for criterion in rule.criteria:
                if criterion.type == "ethValue":
                    criteria.append(
                        SignEvmTransactionCriteriaInner(
                            actual_instance=EthValueCriterion(
                                eth_value=criterion.ethValue,
                                operator=criterion.operator,
                                type="ethValue",
                            )
                        )
                    )
                elif criterion.type == "evmAddress":
                    criteria.append(
                        SignEvmTransactionCriteriaInner(
                            actual_instance=EvmAddressCriterion(
                                addresses=criterion.addresses,
                                operator=criterion.operator,
                                type="evmAddress",
                            )
                        )
                    )
                else:
                    raise ValueError(
                        f"Unknown criterion type {criterion.type} for operation {rule.operation}"
                    )

            rules.append(
                Rule(
                    actual_instance=SignEvmTransactionRule(
                        action=rule.action,
                        operation="signEvmTransaction",
                        criteria=criteria,
                    )
                )
            )
        elif rule.operation == "signSolTransaction":
            criteria = []
            for criterion in rule.criteria:
                if criterion.type == "solAddress":
                    criteria.append(
                        SignSolTransactionCriteriaInner(
                            actual_instance=SolAddressCriterion(
                                addresses=criterion.addresses,
                                operator=criterion.operator,
                                type="solAddress",
                            )
                        )
                    )
                else:
                    raise ValueError(
                        f"Unknown criterion type {criterion.type} for operation {rule.operation}"
                    )

            rules.append(
                Rule(
                    actual_instance=SignSolTransactionRule(
                        action=rule.action,
                        operation="signSolTransaction",
                        criteria=criteria,
                    )
                )
            )
        else:
            raise ValueError(f"Unknown operation {rule.operation}")

    return rules


def map_openapi_rules_to_response_format(openapi_rules: list[Rule]) -> list[RuleType]:
    """Build a properly formatted list of response rules from a list of OpenAPI policy rules.

    Args:
        openapi_rules (List[Rule]): The OpenAPI policy rules to build from.

    Returns:
        List[RuleType]: A list of rules formatted for the response.

    """
    rules = []
    for rule in openapi_rules:
        if rule.actual_instance.operation == "sendEvmTransaction":
            criteria = []
            for criterion in rule.actual_instance.criteria:
                if criterion.actual_instance.type == "ethValue":
                    criteria.append(
                        EthValueCriterionModel(
                            ethValue=criterion.actual_instance.eth_value,
                            operator=criterion.actual_instance.operator,
                        )
                    )
                elif criterion.actual_instance.type == "evmAddress":
                    criteria.append(
                        EvmAddressCriterionModel(
                            addresses=criterion.actual_instance.addresses,
                            operator=criterion.actual_instance.operator,
                        )
                    )
                elif criterion.actual_instance.type == "evmNetwork":
                    criteria.append(
                        EvmNetworkCriterionModel(
                            networks=criterion.actual_instance.networks,
                            operator=criterion.actual_instance.operator,
                        )
                    )
                else:
                    raise ValueError(f"Unknown criterion type {criterion.actual_instance.type}")

            rules.append(
                SendEvmTransactionRuleModel(
                    action=rule.actual_instance.action,
                    criteria=criteria,
                )
            )
        elif rule.actual_instance.operation == "signEvmTransaction":
            criteria = []
            for criterion in rule.actual_instance.criteria:
                if criterion.actual_instance.type == "ethValue":
                    criteria.append(
                        EthValueCriterionModel(
                            ethValue=criterion.actual_instance.eth_value,
                            operator=criterion.actual_instance.operator,
                        )
                    )
                elif criterion.actual_instance.type == "evmAddress":
                    criteria.append(
                        EvmAddressCriterionModel(
                            addresses=criterion.actual_instance.addresses,
                            operator=criterion.actual_instance.operator,
                        )
                    )
                else:
                    raise ValueError(f"Unknown criterion type {criterion.actual_instance.type}")

            rules.append(
                SignEvmTransactionRuleModel(
                    action=rule.actual_instance.action,
                    criteria=criteria,
                )
            )
        elif rule.actual_instance.operation == "signSolTransaction":
            criteria = []
            for criterion in rule.actual_instance.criteria:
                if criterion.actual_instance.type == "solAddress":
                    criteria.append(
                        SolanaAddressCriterionModel(
                            addresses=criterion.actual_instance.addresses,
                            operator=criterion.actual_instance.operator,
                        )
                    )
                else:
                    raise ValueError(f"Unknown criterion type {criterion.actual_instance.type}")

            rules.append(
                SignSolanaTransactionRuleModel(
                    action=rule.actual_instance.action,
                    criteria=criteria,
                )
            )
        else:
            raise ValueError(f"Unknown operation {rule.actual_instance.operation}")

    return rules



# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.list_evm_accounts200_response import ListEvmAccounts200Response

class TestListEvmAccounts200Response(unittest.TestCase):
    """ListEvmAccounts200Response unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ListEvmAccounts200Response:
        """Test ListEvmAccounts200Response
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ListEvmAccounts200Response`
        """
        model = ListEvmAccounts200Response()
        if include_optional:
            return ListEvmAccounts200Response(
                next_page_token = 'eyJsYXN0X2lkIjogImFiYzEyMyIsICJ0aW1lc3RhbXAiOiAxNzA3ODIzNzAxfQ==',
                accounts = [
                    cdp.openapi_client.models.evm_account.EvmAccount(
                        address = '0x742d35Cc6634C0532925a3b844Bc454e4438f44e', 
                        name = 'my-account', 
                        policies = ["123e4567-e89b-12d3-a456-426614174000"], )
                    ]
            )
        else:
            return ListEvmAccounts200Response(
                accounts = [
                    cdp.openapi_client.models.evm_account.EvmAccount(
                        address = '0x742d35Cc6634C0532925a3b844Bc454e4438f44e', 
                        name = 'my-account', 
                        policies = ["123e4567-e89b-12d3-a456-426614174000"], )
                    ],
        )
        """

    def testListEvmAccounts200Response(self):
        """Test ListEvmAccounts200Response"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.sign_evm_transaction_criteria_inner import SignEvmTransactionCriteriaInner

class TestSignEvmTransactionCriteriaInner(unittest.TestCase):
    """SignEvmTransactionCriteriaInner unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SignEvmTransactionCriteriaInner:
        """Test SignEvmTransactionCriteriaInner
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SignEvmTransactionCriteriaInner`
        """
        model = SignEvmTransactionCriteriaInner()
        if include_optional:
            return SignEvmTransactionCriteriaInner(
                type = 'ethValue',
                eth_value = '1000000000000000000',
                operator = 'in',
                addresses = [0x742d35Cc6634C0532925a3b844Bc454e4438f44e, 0x1234567890123456789012345678901234567890]
            )
        else:
            return SignEvmTransactionCriteriaInner(
                type = 'ethValue',
                eth_value = '1000000000000000000',
                operator = 'in',
                addresses = [0x742d35Cc6634C0532925a3b844Bc454e4438f44e, 0x1234567890123456789012345678901234567890],
        )
        """

    def testSignEvmTransactionCriteriaInner(self):
        """Test SignEvmTransactionCriteriaInner"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
